@@ -1,3 +1,4 @@
+import os
 import re,json
 #sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))
 from collections import Iterable
@@ -6,17 +7,18 @@ from manafa.powerProfile.PowerProfile import PowerProfile
 from manafa.utils.Utils import get_pack_dir
 from manafa.utils.dateUtils import convertBatStatTimeToTimeStamp,batStatResetTimeToTimeStamp
 import copy
+from manafa.utils.Logger import log, LogSeverity
 
-DEFAULT_JSON_PATH = get_pack_dir() +"/batteryStats/BatteryStatus.json"
+DEFAULT_JSON_PATH = os.path.join(get_pack_dir(), "batteryStats", "BatteryStatus.json")
 
 
-KNOWN_VALUES=set(
+KNOWN_VALUES = set(
 	"screen.on"
 	#TODO put here all values in https://source.android.com/devices/tech/power/values
 )
 
 
-def areInMinCPUFreq( cpu_freq , possible_cpu_states  ):
+def areInMinCPUFreq(cpu_freq, possible_cpu_states  ):
 	possible_freqs = possible_cpu_states["speeds"] if "speeds" in possible_cpu_states else []
 	if len(possible_freqs)==0:
 		possible_freqs = possible_cpu_states["core_speeds"] if "core_speeds" in possible_cpu_states else []
@@ -33,7 +35,7 @@ def safe_division(a,b):
 
 class BatteryEvent(object):
 	"""docstring for BatteryEvent"""
-	def __init__(self,time=0.0,vals={}):
+	def __init__(self,time=0.0, vals={}):
 		self.time=time
 		self.updates = {}
 		self.currents = {}
@@ -100,7 +102,7 @@ class BatteryEvent(object):
 
 class BatteryStatsParser(object):
 	"""docstring for BatteryStatsParser"""
-	def __init__(self, powerProfile=None , timezone="EST", def_file=DEFAULT_JSON_PATH, android_version=10):
+	def __init__(self, powerProfile=None, timezone="EST", def_file=DEFAULT_JSON_PATH, android_version=10):
 		self.events = []
 		self.definitions = self.loadDefinitionFile(def_file)
 		self.powerProfile = self.loadPowerProfile(powerProfile) if powerProfile is not None else {}
@@ -116,25 +118,26 @@ class BatteryStatsParser(object):
 			return json.load(dff)
 
 	def getDefinitionVal(self,key,val=""):
-		if re.sub(r"\+|\-", "", key) in self.definitions["monoval"]:
+		res = re.sub(r"\+|\-", "", key)
+		if res in self.definitions["monoval"]:
 			return 0 if "-" in key else 1
-		elif re.sub(r"\+|\-", "", key) in self.definitions["trival"]:
+		elif res in self.definitions["trival"]:
 			return key
-		elif re.sub(r"\+|\-", "", key) in self.definitions["numerical"]:
+		elif res in self.definitions["numerical"]:
 			return val
-		elif re.sub(r"\+|\-", "", key) in self.definitions["nominal"]:
+		elif res in self.definitions["nominal"]:
 			return self.definitions["nominal"][str(key)][val]
 		return None
 
-	def isTrival(self,key):
+	def isTrival(self, key):
 		return re.sub(r"\+|\-", "", key) in self.definitions["trival"]
 
-	def parseStates(self,states):
+	def parseStates(self, states):
 		accum=False
 		accumulator = ""
 		latest_state=None
 		events={}
-		for state in re.sub(r"^ ","",states).split(" "):
+		for state in re.sub(r"^ ", "", states).split(" "):
 			#print("->" +state)
 			if accum:
 				accumulator+=state
@@ -152,10 +155,9 @@ class BatteryStatsParser(object):
 					val = state.split("=")[1]
 					state = self.getDefinitionVal(key,val)
 					if self.isTrival(key):
-						#print("TODO: split in trival")
 						#print("%s - %s -%s" %(key,val.split(":")[0],val.split(":")[1]))
 						#return key,val.split(":")[0],val.split(":")[1]
-						events[key]= {"val": val.split(":")[0] , "val2": val.split(":")[1]}
+						events[key] = {"val": val.split(":")[0], "val2": "".join(val.split(":")[1:])}
 					else:
 						#print("%s = %s" %(key,state))
 						events[key]=state
@@ -187,13 +189,14 @@ class BatteryStatsParser(object):
 					#print(epochToDate(self.start_time))
 			else:
 				# TODO Handle DcpuStats and DpstStats
-				print("linha invalida" + line)
+				#print(line)
+				log("Unrecognized patter in line of batstats file", LogSeverity.WARNING)
 
-	def addUpdate(self, time,bat_events):
-		if len(self.events)==0:
-			bt = BatteryEvent( time,bat_events )
+	def addUpdate(self, time, bat_events):
+		if len(self.events) == 0:
+			bt = BatteryEvent(time, bat_events)
 			self.estimateCurrentConsumption(bt)
-			self.events.append(bt )
+			self.events.append(bt)
 		else:	
 			last_added = self.events[-1]
 			if last_added.time == time:
@@ -210,12 +213,67 @@ class BatteryStatsParser(object):
 
 	def estimateCurrentConsumption(self,bt_event):
 		power={}
-		for p,v in self.powerProfile.components.items():
-			st = self.determinateComponentCurrent(bt_event,p,v)
+		for p, v in self.powerProfile.components.items():
+			st = self.determinateComponentCurrent(bt_event, p, v)
 			power[p]=st
 			#print("%s %s" %(p , str(st)))
 		bt_event.currents=power
-		
+
+	def getClosestPair(self, time):
+		lasti = 0
+		for i, x in enumerate(self.events):
+			if x.time > time:
+				return lasti, i
+			lasti = i
+		return lasti, lasti
+
+	def getEventsInBetween(self,start_time, end_time):
+		c_beg_bef, c_beg_aft = self.getClosestPair(start_time)
+		metrics = {}  # {'health: [(event_state,start,end, pctage_duration)], ..}
+		prev_time = self.events[c_beg_aft].time if len(self.events) > 0 else start_time
+		fst_time = prev_time
+		for ev in self.events[c_beg_aft:]:
+			if ev.time > end_time:
+				break
+			for kup, upval in ev.updates.items():
+				metrics[kup] = [] if kup not in metrics else metrics[kup]
+				contains_update_val = metrics[kup][-1][0] == upval if len(
+					metrics[kup]) > 0 else None  # next(filter(lambda x: x[0] == upval, metrics[kup]), None)
+				if not contains_update_val:
+					if len(metrics[kup]) > 0:
+						# update previous state
+						duration_pctage = 100 * ((ev.time - metrics[kup][-1][1]) / (end_time - start_time))
+						metrics[kup][-1] = (
+						metrics[kup][-1][0], metrics[kup][-1][1], ev.time, duration_pctage)  # last_time?
+					init_time = start_time if fst_time == ev.time else ev.time
+					duration_pctage = 100 * ((end_time - init_time) / (end_time - start_time))
+					metrics[kup].append((upval, init_time, end_time, duration_pctage))
+			# concs
+			for cup, cupval in ev.concurrentUpdates.items():
+				metrics[cup] = [] if cup not in metrics else metrics[cup]
+				# detect pushes
+				for c_i, cval in enumerate(cupval):
+					#is_in_metrics = metrics[cup][-1][0]['val'] == cval['val'] and metrics[cup][-1][0]['val2'] == cval['val2']  if len(metrics[cup]) > 0 else False
+					is_in_metrics = next(filter(lambda x: x[0]['val'] == cval['val'] and x[0]['val2'] == cval['val2'] and x[2] == end_time ,metrics[cup]), None)
+					if not is_in_metrics:
+						# new state, add
+						init_time = start_time if prev_time == ev.time else ev.time
+						duration_pctage = 100 * ((end_time - init_time) / (end_time - start_time))
+						metrics[cup].append((cval, init_time, end_time, duration_pctage))
+				# detect pops
+				for i, val_of_metrics in enumerate(metrics[cup]):
+					this_ev_contains_val = next(filter(
+						lambda t: t['val'] == val_of_metrics[0]['val'] and t['val2'] == val_of_metrics[0]['val2'],
+						cupval), None)
+					if not this_ev_contains_val:
+						# was popped, update
+						already_popped = val_of_metrics[2] != end_time
+						if not already_popped:
+							prev_start_time = val_of_metrics[1]
+							duration_pctage = 100 * ((ev.time - prev_start_time) / (end_time - start_time))
+							metrics[cup][i] = (val_of_metrics[0], prev_start_time, ev.time, duration_pctage)
+			prev_time = ev.time
+		return metrics
 
 	def getCPUSamplesInBetween(self,start_time, end_time ):
 		l =[]
@@ -230,15 +288,14 @@ class BatteryStatsParser(object):
 				l.append(pair)
 				last_time= x.time
 			last_ev = x
-		# 
 		last_delta = end_time - last_time
 		last_state = last_ev.currents["cpu"]
 		last_voltage = last_ev.getVoltageValue() #float(last_ev.updates["volt"])
-		last_pair = ( last_delta, last_state,last_voltage)
+		last_pair = (last_delta, last_state, last_voltage)
 		l.append(last_pair)
 		return l
 
-	def determinateComponentCurrent(self,bt_event,comp_name, possible_states):
+	def determinateComponentCurrent(self, bt_event,comp_name, possible_states):
 		current = 0.0
 		curravg=0
 		avg_ct=0
@@ -246,8 +303,8 @@ class BatteryStatsParser(object):
 		if comp_name == "screen" and "screen" in bt_event.updates:
 				on_current = possible_states["on"]
 				brightness_level = bt_event.updates["brightness"] if "brightness" in bt_event.updates else 1
-				relative_full_current = ( brightness_level * possible_states["full"] / ( len( self.definitions["nominal"]["brightness"] ) -1) ) 
-				current+= on_current + relative_full_current 
+				relative_full_current = (brightness_level * possible_states["full"] / (len( self.definitions["nominal"]["brightness"]) -1) )
+				current+= on_current + relative_full_current
 
 		elif comp_name == "ambient" and "screen_doze" in bt_event.updates:
 				# power profile might have a defined value for ambient/doze screen consumpri
@@ -327,16 +384,16 @@ class BatteryStatsParser(object):
 		
 		#bluetooth
 		elif comp_name == "bluetooth":
-			if self.android_version<7 or "controller" not in possible_states:
+			if self.android_version < 7 or "controller" not in possible_states:
 				#account blue on and active vals
-				if "ble_scan" in  bt_event.updates:
-					current+=  possible_states["active"] if "active" in possible_states else 0
-					current+=  possible_states["on"] if "on" in possible_states else 0
-				elif "bluetooth" in  bt_event.updates:
-					current+=  possible_states["on"] if "on" in possible_states else 0
+				if "ble_scan" in bt_event.updates:
+					current += possible_states["active"] if "active" in possible_states else 0
+					current+= possible_states["on"] if "on" in possible_states else 0
+				elif "bluetooth" in bt_event.updates:
+					current += possible_states["on"] if "on" in possible_states else 0
 			elif "controller" in possible_states:
-				current += possible_states["controller"]["idle"] if ( "idle" in possible_states["controller"] ) else 0
-				if "ble_scan" in  bt_event.updates:
+				current += possible_states["controller"]["idle"] if ("idle" in possible_states["controller"]) else 0
+				if "ble_scan" in bt_event.updates:
 					if "tx" in possible_states["controller"]:
 						curravg+=possible_states["controller"]["tx"]
 						avg_ct+=1
@@ -367,13 +424,12 @@ class BatteryStatsParser(object):
 				curravg=0
 				avg_ct=0
 				if "tx" in possible_states["controller"]:
-					on_vals =  possible_states["controller"]["tx"] if "tx" in possible_states["controller"] else [] 
+					on_vals = possible_states["controller"]["tx"] if "tx" in possible_states["controller"] else []
 					signal_stren = bt_event.updates["phone_signal_strength"] if "phone_signal_strength" in bt_event.updates else 0
 					if signal_stren > len(on_vals) and len(on_vals)>0:
 						curravg+= on_vals[-1]
-					elif  len(on_vals)>0:
-						curravg+= on_vals[signal_stren]
-
+					elif len(on_vals) > 0:
+						curravg += on_vals[signal_stren]
 					avg_ct+=1
 				if "rx" in possible_states["controller"]:
 					curravg+=possible_states["controller"]["rx"]
